/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __CLUSTER_MODEL_H__
#define __CLUSTER_MODEL_H__

#include "Matrix.h"
#include "DataSet.h"

template <class T> class IClusterModel
{
	protected:

		int m_k;

		CDataSet<T>* m_dataSet;

		CMatrix<T> m_centers;

	public:

		IClusterModel( CDataSet<T>* dataSet, int k ) :
		    m_dataSet( dataSet ),
			m_k( k ),
			m_centers( k, dataSet->nFeat() )
		{
		}

		CDataSet<T>& dataSet() {
			return *m_dataSet;
		}

		int k() const {
			return m_k;
		}

		CMatrix<T>& centers() const {
			return m_centers;
		}

		CMatrix<T>& centers() {
			return m_centers;
		}

		virtual void cluster( int maxIter = 1000 ) = 0;
};

#endif