/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __DATASET_H__
#define __DATASET_H__

#include <string>
#include <fstream>

#include "Matrix.h"

template <class T> class CDataSet
{
	protected:

		CMatrix<T> m_x;
	
		CVector<int> m_y;

	public:

		CDataSet() {
		}

		CDataSet( std::string const& fname ) {
			rawImport( fname );
		}

		~CDataSet() {
			release();
		}

		void release() {
			m_x.release();
			m_y.release();
		}

		CMatrix<T>& x() {
			return m_x;
		}

		CVector<int>& y() {
			return m_y;
		}

		int nInst() const {
			return m_x.nRows();
		}

		int nFeat() const {
			return m_x.nCols();
		}

		bool rawImport( std::string const& fname );
};

template <class T> bool CDataSet<T>::rawImport( std::string const& fname )
{
	std::ifstream ifs( fname.c_str() );

	if ( !ifs.is_open() ) {
		return false;
	}

	char buffer[ 4096 ];
	ifs.getline( buffer, 4096 );
	string line( str_trim( buffer ) );
	if ( line.empty() ) {
		return false;
	}
	std::vector<std::string> tokens = str_split( line );
	int nCols = tokens.size() - 1;
	int nRows = 1;
	bool done = false;
	while ( !done ) {
		if ( ifs.eof() ) {
			done = true;
		} else {
			ifs.getline( buffer, 4096 );
			line = str_trim( buffer );
			if ( line.empty() ) {
				done = true;
			} else {
				nRows++;
			}
		}
	}
	
	ifs.clear();

	ifs.seekg( 0, std::ios::beg );

	m_x.alloc( nRows, nCols );
	m_y.alloc( nRows );

	int r, c;
	int* ptr_y = m_y.dataPtr();
	T* ptr_x = m_x.dataPtr();
	for ( r = 0; r < m_x.nRows(); r++ ) {
		double y;
		ifs >> y;
		*ptr_y = static_cast<int>( y );
		++ptr_y;
		for ( c = 0; c < m_x.nCols(); c++ ) {
			double x;
			ifs >> x;
			*ptr_x = static_cast<T>( x );
			++ptr_x;
		}
	}

	ifs.close();
	
	return true;
}

#endif