/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __INIT_MODEL_KKZ_H__
#define __INIT_MODEL_KKZ_H__

#include <cstring>

#include <limits>

#include "InitModel.h"

template <class T> class CInitModelKKZ : public IInitModel<T>
{
	protected:

	public:

		CInitModelKKZ( IClusterModel<T>* clusterModel ) :
		    IInitModel<T>( clusterModel )
		{
		}

		virtual bool init() {
			CMatrix<T>& x = clusterModel()->dataSet().x();

			CMatrix<T>& centers = clusterModel()->centers();

			const int n = x.nRows();
			const int d = x.nCols();

			CMatrix<T> dist( clusterModel()->k(), n, true, std::numeric_limits<T>::max() );

			T* distPtr = dist.dataPtr();

			x.dist( CVector<T>( d, true ) );

			CVector<T>( n, distPtr ) = x.dist( CVector<T>( d, true ) );

			std::pair<int, T> max = CVector<T>( n, distPtr ).max();

			T* centPtr = centers.dataPtr();

			CVector<T>( d, centPtr ) = CVector<T>( d, x[ max.first ] );

			int k = 1;
			while ( k < clusterModel()->k() ) {
				CVector<T>( n, distPtr ) = x.dist( CVector<T>( d, centPtr ) );

				CIndexValuePair<T> min = dist.min( 1 );

				std::pair<int, T> max = min.val().max();

				CVector<T>( d, centPtr + d ) = CVector<T>( d, x[ max.first ] );

				++k;

				centPtr += d;
				distPtr += n;
			}

			return true;
		}
};

#endif