/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __INIT_MODEL_RANDOM_H__
#define __INIT_MODEL_RANDOM_H__

#include "InitModel.h"

template <class T> class CInitModelRandom : public IInitModel<T>
{
	protected:

	public:

		CInitModelRandom( IClusterModel<T>* clusterModel ) :
		    IInitModel<T>( clusterModel )
		{
		}

		virtual bool init() {
			IClusterModel<T>* model = m_clusterModel;
			CMatrix<T>& x = model->dataSet().x();
			const int n = x.nRows();
			const int d = x.nCols();
			CMatrix<T>& centers = model->centers();

			CVector<int> rnd( 1, n );
			std::random_shuffle( rnd.dataPtr(), rnd.dataPtr() + n ); 

			int i;
			T* dst = centers.dataPtr();			
			for ( i = 0; i < model->k(); ++i, dst += d ) {
				CVector<T>( d, dst ) = CVector<T>( d, x[ rnd[ i ] ] ); 
			}

			return true;
		}
};

#endif