/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __INIT_MODEL_SELECTION_H__
#define __INIT_MODEL_SELECTION_H__

#include "InitModel.h"

template <class T> class CInitModelSelection : public IInitModel<T>
{
	protected:

		CVector<int> m_indices;

		CVector<int>& indices() {
			return m_indices;
		}

	public:

		CInitModelSelection( IClusterModel<T>* clusterModel, CVector<int> const& indices ) :
		    IInitModel<T>( clusterModel ),
			m_indices( indices )
		{
		}

		virtual bool init() {
			IClusterModel<T>* model = clusterModel();
			const int d = model->dataSet().nFeat();

			int i;
			for ( i = 0; i < model->k(); ++i ) {
				CVector<T>( d, model->centers()[ i ] ) = CVector<T>( d, model->dataSet().x()[ indices()[ i ] ] );
			}

			return true;
		}
};

#endif