/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __KMEANS_MODEL_H__
#define __KMEANS_MODEL_H__

#include "ClusterModel.h"

template <class T> class CKMeansModel : public IClusterModel<T>
{
	protected:

	public:

		CKMeansModel( CDataSet<T>* dataSet, int k ) :
		    IClusterModel( dataSet, k )
		{
		}

		virtual void cluster( int maxIter = 1000 ) {
			const int n = dataSet().nInst();
			const int d = dataSet().nFeat();

			CMatrix<T> mDist( k(), n );

		    CIndexValuePair<T> vMin( n );
			CVector<int> count( k() );
			CVector<T> vDist( n, mDist.dataPtr() );

			cout << centers() << endl;

			int iter = 0;
			bool done = false;
			while ( !done ) {
				int c;
				vDist.setDataPtr( mDist.dataPtr() );
				for ( c = 0; c < k(); ++c , vDist.incDataPtr( n ) ) {
					dataSet().x().dist( CVector<T>( d, centers()[ c ] ), vDist );
				}

				// cout << vDist << endl;

				mDist.min( 1, vMin );

				// cout << vMin.idx().size() << endl;

				// cout << vMin.idx() << endl;
				// cout << vMin.val() << endl;

				for ( c = 0; c < k(); ++c ) {
					CVector<T>( d, centers()[ c ] ).zeros();
				}
				count.zeros();

				int i;
				for ( i = 0; i < n; ++i ) {
					const int idx = vMin.idx()[ i ];
					CVector<T>( d, centers()[ idx ] ) += ( CVector<T>( d, dataSet().x()[ i ] ) );
					++count[ idx ];
				}

				// cout << count << endl;

				for ( c = 0; c < m_k; ++c ) {
					if ( count[ c ] ) {
						CVector<T>( d, centers()[ c ] ) *= ( 1.0f / count[ c ] );
					}
				}

				cout << "iteration " << iter << ":" << endl;
				cout << centers() << endl;

				++iter;

				done = iter == maxIter;
			}
		}
};

#endif