/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __MEM_UTILS_H__
#define __MEM_UTILS_H__

#include <cstring>

#include "sseUtils.h"

template <typename T> inline T*& lvNullPtr() 
{
	static T* ptr = NULL;
	return ptr;
}

template <class T> class CMemory
{
	public:

		static T* alloc() {
			return new T;
		}
		
		static T* alloc( int nElements ) {
			return new T[ nElements ];
		}

		static void release( T*& item, bool array = false ) {
			if ( array ) {
				delete [] item;
			} else {
				delete item;
			}
			item = NULL;
		}
};

template <> class CMemory<AlignedMemory16f>
{
	public:

		static AlignedMemory16f* alloc() {
			return ( AlignedMemory16f* )_aligned_malloc( sizeof( AlignedMemory16f ), 16 );
		}
		
		static AlignedMemory16f* alloc( int nElements ) {
			return ( AlignedMemory16f* )_aligned_malloc( nElements * sizeof( AlignedMemory16f ), 16 );
		}

		static void release( AlignedMemory16f*& item ) {
			_aligned_free( item );
			item = NULL;
		}
};

#endif