/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __SORT_UTILS_H__
#define __SORT_UTILS_H__

#include "Vector.h"

template <class T> void quickSort( CVector<T>& val, CVector<int>& idx )
{
    int idxL = 0;
    int idxR = val.size() - 1;

    bool done = false;
    T pivot = val[ val.size() >> 1 ];
    while ( !done ) {
        while ( val[ idxL ] < pivot ) ++idxL;
        while ( val[ idxR ] > pivot ) --idxR;
        if( idxL > idxR ) {
            done = true;
        } else {
            std::swap( val[ idxL ], val[ idxR ] );
            std::swap( idx[ idxL ], idx[ idxR ] );
            ++idxL;
            --idxR;
            done = idxL > idxR;
        }
    }    
    if ( idxR > 0 ) {
		CVector<T> auxVal( idxR + 1, &val[ 0 ] );
        CVector<int> auxIdx( idxR + 1, &idx[ 0 ] );
        quickSort<T>( auxVal, auxIdx );
    }
    if ( idxL < val.size() - 1 ) {
		CVector<T> auxVal( val.size() - idxL, &val[ idxL ] );
		CVector<int> auxIdx( val.size() - idxL, &idx[ idxL ] );
        quickSort<T>( auxVal, auxIdx );
    }
}

template <class T> CSortResult<T> quickSort( CVector<T> const& val )
{
    CSortResult<T> sorted( val.size() );

    sorted.val() = val;
    sorted.idx() = CVector<int>( 0, val.size() );

    quickSort<T>( sorted.val(), sorted.idx() );

    return sorted;
}

#endif