/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#include "StringUtils.h"

#include <string>
	using std::string;
#include <vector>
	using std::vector;
#include <sstream>
	using std::stringstream;

string str_replace( const string& str, const string& from, const string& to )
{
	string result( str );

	size_t idx = str.find( from );

	if ( idx != string::npos ) {
		return result.replace( idx, from.length(), to );
	} else {
		return result;
	}
}

string str_ltrim( const string& str, const string& delims )
{
    size_t idx = str.find_first_not_of( delims );

    if ( idx != string::npos ) {
        return str.substr( idx, str.length() - idx );
    } else {
        return string( "" );
    }
}

string str_rtrim( const string& str, const string& delims )
{
    size_t idx = str.find_last_not_of( delims );

    if ( idx != string::npos ) {
        return str.substr( 0, idx + 1 );
    } else {
        return string( "" );
    }
}

string str_trim( const string& str, const string& delims )
{
    return str_ltrim( str_rtrim( str, delims ), delims );
}

vector<string> str_split( const string& str, const string& delims )
{
    vector<string> list;

    string aux = str_trim( str, delims );

    bool done = false;
    while ( !done ) {
        size_t idx = aux.find_first_of( delims );
        if ( idx != string::npos ) {
            list.push_back( aux.substr( 0, idx ) );
            if ( idx < aux.length() - 1 ) {
                aux = str_ltrim( aux.substr( idx + 1, aux.length() - ( idx + 1 ) ), delims );
            }
        } else {
            if ( !aux.empty() ) {
                list.push_back( aux );
            }
            done = true;
        }
    }

    return list;
}

void str_fsplit( const string& str, string* drive, string* path, string* fname, string* extension )
{
	string aux( str );
	size_t idx = aux.find_first_of( "\\" );
	if ( drive ) {
		*drive = aux.substr( 0, idx );
	}
	aux = aux.substr( idx, aux.length() - idx );
	idx = aux.find_last_of( "\\" );
	if ( path ) {
		*path = aux.substr( 0, idx );
	}
	aux = aux.substr( idx + 1, aux.length() - ( idx - 1 ) );
	idx = aux.find( "." );
	if ( idx != string::npos ) {
	    if ( fname ) {
	        *fname = aux.substr( 0, idx );
	    }
	    if ( extension ) {
	        *extension = aux.substr( idx + 1, aux.length() - ( idx + 1 ) );
	    }
	} else {
	    if ( fname ) {
	        *fname = aux;
	    }
	    if ( extension ) {
	        *extension = "";
	    }
	}
}

void str_fsplit( const string& fname, FileInfo& info )
{
	str_fsplit( fname, &info.drive, &info.path, &info.name, &info.ext );
}
