/*
 * Copyright (C) 2013 Philippe Komma
 * 
 * This file is part of k-Means: A C++ clustering library 
 * 
 * k-Means is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with k-Means. If not, see <http://www.gnu.org/licenses/>.
 */

#include "sseUtils.h"

#include <intrin.h>

#include <iostream>
	using std::cout;
	using std::endl;
#include <string>
	using std::string;

void CSSEUtils::info( CPUInfo& info  )
{
	int reg[4];
	__cpuid( reg, 0x0000001 );

	info.fpuAvail   = ( reg[ 3 ] & ( 1ul <<  0 ) ) != 0;
	info.sseAvail   = ( reg[ 3 ] & ( 1ul << 23 ) ) != 0;
	info.sseAvail   = ( reg[ 3 ] & ( 1ul << 25 ) ) != 0;
	info.sse2Avail  = ( reg[ 3 ] & ( 1ul << 26 ) ) != 0;
	info.sse3Avail  = ( reg[ 2 ] & ( 1ul <<  0 ) ) != 0;
	info.sse41Avail = ( reg[ 2 ] & ( 1ul << 19 ) ) != 0;
	info.sse42Avail = ( reg[ 2 ] & ( 1ul << 20 ) ) != 0;
	info.aesAvail   = ( reg[ 2 ] & ( 1ul << 25 ) ) != 0;
	info.avxAvail   = ( reg[ 2 ] & ( 1ul << 28 ) ) != 0;
	info.htAvail    = ( reg[ 3 ] & ( 1ul << 28 ) ) != 0;
}

inline string is_available( bool entry )
{
	if ( entry ) {
		return string( "available" );
	} else {
		return string( "NOT available" );
	}
}

std::ostream& operator<<( std::ostream& os, const CSSEUtils::CPUInfo& info )
{
	os << "CPU information" << endl;
	os << "  * fpu....." << is_available( info.fpuAvail ) << endl;
	os << "  * sse....." << is_available( info.sseAvail ) << endl;
	os << "  * sse2...." << is_available( info.sse2Avail ) << endl;
	os << "  * sse3...." << is_available( info.sse3Avail ) << endl;
	os << "  * sse41..." << is_available( info.sse41Avail ) << endl;
	os << "  * sse42..." << is_available( info.sse42Avail ) << endl;
	os << "  * aes....." << is_available( info.aesAvail ) << endl;
	os << "  * avx....." << is_available( info.avxAvail ) << endl;
	os << "  * ht......" << is_available( info.htAvail ) << endl;
	
	return os;
}
